@@ -5,7 +5,7 @@
  *
  * Based upon mec2.h
  * 
- * Copyright (C) 2002, Digium, Inc.
+ * Copyright (C) 2002-2012, Digium, Inc.
  *
  * Additional background on the techniques used in this code can be found in:
  *
@@ -44,9 +44,6 @@
 static int debug;
 static int aggressive;
 
-#define module_printk(level, fmt, args...) printk(level "%s: " fmt, THIS_MODULE->name, ## args)
-#define debug_printk(level, fmt, args...) if (debug >= level) printk(KERN_DEBUG "%s (%s): " fmt, THIS_MODULE->name, __FUNCTION__, ## args)
-
 /* Uncomment to provide summary statistics for overall echo can performance every 4000 samples */ 
 /* #define MEC2_STATS 4000 */
 
@@ -148,9 +145,11 @@ static void echo_can_free(struct dahdi_chan *chan, struct dahdi_echocan_state *e
 static void echo_can_process(struct dahdi_echocan_state *ec, short *isig, const short *iref, u32 size);
 static int echo_can_traintap(struct dahdi_echocan_state *ec, int pos, short val);
 static void echocan_NLP_toggle(struct dahdi_echocan_state *ec, unsigned int enable);
+static const char *name = "KB1";
+static const char *ec_name(const struct dahdi_chan *chan) { return name; }
 
 static const struct dahdi_echocan_factory my_factory = {
-	.name = "KB1",
+	.get_name = ec_name,
 	.owner = THIS_MODULE,
 	.echocan_create = echo_can_create,
 };
@@ -160,7 +159,6 @@ static const struct dahdi_echocan_features my_features = {
 };
 
 static const struct dahdi_echocan_ops my_ops = {
-	.name = "KB1",
 	.echocan_free = echo_can_free,
 	.echocan_process = echo_can_process,
 	.echocan_traintap = echo_can_traintap,
@@ -266,7 +264,7 @@ static inline short get_cc_s(echo_can_cb_s *cb, int pos)
 
 static inline void init_cc(struct ec_pvt *pvt, int N, int maxy, int maxu)
 {
-	void *ptr = pvt;
+	char *ptr = (char *) pvt;
 	unsigned long tmp;
 
 	/* Double-word align past end of state */
@@ -281,9 +279,9 @@ static inline void init_cc(struct ec_pvt *pvt, int N, int maxy, int maxu)
 	pvt->beta2_i = DEFAULT_BETA1_I;
   
 	/* Allocate coefficient memory */
-	pvt->a_i = ptr;
+	pvt->a_i = (int *) ptr;
 	ptr += (sizeof(int) * pvt->N_d);
-	pvt->a_s = ptr;
+	pvt->a_s = (short *) ptr;
 	ptr += (sizeof(short) * pvt->N_d);
 
 	/* Reset Y circular buffer (short version) */
@@ -723,7 +721,8 @@ static int __init mod_init(void)
 		return -EPERM;
 	}
 
-	module_printk(KERN_NOTICE, "Registered echo canceler '%s'\n", my_factory.name);
+	module_printk(KERN_NOTICE, "Registered echo canceler '%s'\n",
+		      my_factory.get_name(NULL));
 
 	return 0;
 }
